/*
 * TestRes 1.0
 * ---------------------------------------------------------
 * (c)1998 , Daniel Lundin , Mediacenter (http://www.umc.se/)
 * Contact: Daniel Lundin <guru@umc.se>
 *
 * Usage:  restest [res_file res_id]
 *
 * A very simple program to demonstrate how to load an
 * archived flattened BBitmap from a resource file,
 * that is using what is produced with img2res.
 *
 */
 
 
#define APP_SIGNATURE "application/x-vnd.dpp-ResTest"


// A BView derived class that holds  and displays
// a BBitmap.

class MyView : public BView
{
	public:
		MyView(BRect frame, BBitmap *bmap);
		~MyView();
		void Draw(BRect updateRect);
		
	private:
		BBitmap	*themap;
};


// Minimalistic window for demonstrating purpose

class MyWin : public BWindow
{
	public:
		MyWin(const char* resfile, int32 resid);
		~MyWin();
		bool 			QuitRequested(void);
	private:
		MyView	*mview;
};



/* ---------------------------------- */
/* -------------- MAIN -------------- */
/* ---------------------------------- */

int main(int argc, char *argv[])
{
	char			*resfile="kittycat.rsrc";	// Default resfile
	int32			resid=1;				// Default resid
	
	BApplication	*app=new BApplication(APP_SIGNATURE);		// Init application
	

	if( argc>2 ) {
		resfile=argv[1];	
		resid=strtol(argv[2],NULL,10);
	}

	BWindow		*mywin= new MyWin(resfile,resid);
	app->Run();
	
	delete app;
	return 0;						
}
 


/***
 ***
 *** MyWindow
 ***
 ***/


// Window constructor.
// Loads the requested resource, unflattens the bitmap
// and creates a MyView with the resulting BBitmap.

 MyWin::MyWin(const char* resfile, int32 resid) :
 			 BWindow(		BRect(50,50,100,100),
						"ResTest",
						B_TITLED_WINDOW,
						NULL
					)
{
	BResources res; 
   	status_t err; 
   	
   	size_t  reslength;
   	const char * resbuf;
   	BMessage archive;
	BBitmap	*themap;

	// 	Open resource file

	BFile file(resfile,B_READ_ONLY);  

   	if ( (file.InitCheck() || (err = res.SetTo(&file) ))  == B_NO_ERROR)  {

		// Find and load resource
   		resbuf=(char *)res.FindResource(	B_RAW_TYPE,
   							resid,
   							&reslength );
   							
   		if( resbuf ) {
   		
			// Inflate and unarchive BBitmap
	   		archive.Unflatten(resbuf);	
   			themap=new BBitmap(&archive);

			// Resize window to bitmap dimenstions
  			ResizeTo(	themap->Bounds().Width(),
  					themap->Bounds().Height()
  					);		
  	
			// Construct a MyView with the BBitmap 				
	  		mview=new MyView( Bounds(), themap);
  		

			// Attach the MyView to window
  			Lock();
  			AddChild(mview);
	  		Unlock();
  			
  			// Let's go...	
			Show();
		} else {
	   		printf("Error while reading resource from file.\n");
		}
		    		
   	} else {	// Error
   		printf("Error opening resource file.\n");
   		exit(1);
   	}

}


// Window destructor.
// Cleans up and deletes the MyView
MyWin::~MyWin()
{
	Lock();
	RemoveChild(mview);
	delete mview;
	Unlock();
}



// Window closed
bool MyWin::QuitRequested(void)
{
	// Tell application to quit
	be_app->PostMessage(B_QUIT_REQUESTED);	
	return TRUE;
}





/***
 ***
 *** MyView
 ***
 ***/



// Constructor that copies the bitmap pointer.
// This of course should be done in a cleaner way,
// but the time, the time.
 MyView::MyView(BRect frame, BBitmap *bmap) : 
 				BView(	frame,
						"ResTest",
						B_FOLLOW_ALL_SIDES,
						B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE
					)
{
	themap=bmap;
}


// View destructor. Cleans up after the view.
MyView::~MyView()
{
	delete themap;
}


// Draw the bitmap
void MyView::Draw(BRect updateRect)
{
	if ( Window()->Lock() )
	{
		DrawBitmap(themap,Bounds());
		Window()->Unlock();
	}
}
 




