/*
 * Img2res 1.0
 * ---------------------------------------------------------
 * (c)1998 , Daniel Lundin , Mediacenter (http://www.umc.se/)
 * Contact: Daniel Lundin <guru@umc.se>
 *
 * Img2res translates an imagefile and puts it into a resource 
 * file as an archived BBitmap in a flattened BMessage.
 *
 */
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <TranslationKit.h>

#define APP_SIGNATURE "application/x-vnd.dpp-img2res"

#define VERSION "1.0"

#define USAGE "\nImg2res " VERSION \
	" - Daniel Lundin <guru@umc.se>\n\n" \
	"Usage: img2res inputfile resfile [res_id [res_name]]\n" \
	"   inputfile - Image file\n" \
	"     resfile - Resourcefile\n" \
	"      res_id - Resource ID\n" \
	"    res_name - Resource name\n" \
	"\n" \
	"If resfile does not exist, the file will be created.\n" \
	"If res_id is  not given, a unique ID will be used." \
	"\n"

#define DEFAULT_RESNAME "Bitmap Resource"



BBitmap *FetchBitmap(char *filename);
status_t SaveRes(char *resfilename, BBitmap *bmap, int32 resid, const char *resname);

int main(int32 argc, char **argv)
{
	char 			*infile_name;	// Input imagefile name
	char 			*resfile_name;	// Resource file
	char 			*resname;		// Resource name
	int32 			resid;			// Resource ID
	BBitmap 		*themap;		// The resulting bitmap
	status_t		err=B_OK;		// Result code

	BApplication	app(APP_SIGNATURE);		// Init application

	if( argc<3 || argc>5 ) {	// No valid args?	
		printf(USAGE);		// Then print usage info

	} else {
	
		infile_name=argv[1];	// Name of file to import
		resfile_name=argv[2];	// Name of resfile

		// Resource ID requested 
		if( argc > 3 )
			resid=strtol(argv[3],NULL,10);
		else
			resid=-1;	// "Magic" number

		// Resource Name requested
		if( argc==5 )
			resname=argv[4];
		else
			resname=DEFAULT_RESNAME;	// Default resource name


		// Read input file 
		themap=FetchBitmap(infile_name);

		if( themap ) {
			// Read and translate input file
			err=SaveRes(resfile_name,themap,resid,resname);
			delete themap;
		} else {
			printf("Error reading inputfile\n");
			err=B_ERROR;	// Lets call this an error upon exit
		}
		
	}

	return err;
}
 

BBitmap *FetchBitmap(char *filename) 
{ 
	BBitmap *result = NULL; 						// The translated bitmap
	BBitmapStream *bstream=new BBitmapStream(); 		// Bitstream 
	BTranslatorRoster *roster = BTranslatorRoster::Default();	// Get the default roster
	BFile ifile(filename, B_READ_ONLY); 				// Open bitmap file

	// Check that file was opened nicely	
	if ( ifile.InitCheck()!=B_OK ) {
		printf("Error when opening input\n");
		return NULL;
	} else {	
	
		// Translate inputfil 
		roster->Translate(&ifile, NULL, NULL, bstream, 
			B_TRANSLATOR_BITMAP);
	
		// Hand us a convenient BBitmap
		bstream->DetachBitmap(&result); 

		if( result )
			printf(	"     Input file: %s\n"
					"Bitmap bytesize: %d\n" 
					"   Bitmap width: %.0f\n"
					"  Bitmap height: %.0f\n",
					filename,
					result->BitsLength(),
					result->Bounds().Width(),
					result->Bounds().Height()
				);

		delete bstream;
		return result; 
	}
}



status_t SaveRes(char *resfilename, BBitmap *bmap, int32 resid, const char *resname)
{
	status_t	err=B_OK;	// Error result
	type_code	restype=B_RGB_32_BIT_TYPE;
	BResources	res; 
	BFile 		file(resfilename, B_READ_WRITE | B_CREATE_FILE); 
	BMessage 	archive;	// Archive message for the BBitmap
	BMallocIO 	obuf;		// Flat buffer for archived message
	ssize_t	obufsize;	// Flat buffer size

	type_code	rtype;	//
	int32		rid;		//  Tmp-vars used for stepping through
	char *	rname;	//  resources and determining unique ID
	size_t	rlength;	//
	

	// Open resourcefile, create new if not existing
		if ((err = res.SetTo(&file)) != B_NO_ERROR) {
		if ((err = res.SetTo(&file, true)) != B_NO_ERROR) {
			printf("Error opening resourcefile [%i]\n",err);
			return err;
		} else {
			resid=1;
		}
	 } else {

	 	// If no resource ID given, so we should find a valid new ID to use 
		if( resid == -1 ) 
			for ( int32 i=0; 
				res.GetResourceInfo(	i, &rtype, &rid,&rname, &rlength );
				i++ ) 
	 			if( rid >= resid )
	 				resid=rid+1;
	}

	printf("    Resource ID: %d\n", resid);

	
	// Create archived message from BBitmap
	bmap->Archive(&archive);
	archive.Flatten(&obuf,&obufsize);
	printf("  Archived size: %d\n", obufsize);


	// Add resource to file		
	err=res.AddResource(	B_RAW_TYPE,
						resid,
						obuf.Buffer(),
						obuf.BufferLength(),
						resname 
					);
	if( err != B_OK )
		printf("Error writing resource\n");
		
								
	return err;	
}

